@@ -1,24 +1,49 @@
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-from twisted.trial import unittest
-import string, random, copy
+"""
+Tests for various parts of L{twisted.web}.
+"""
+
 from cStringIO import StringIO
 
-from twisted.web import server, resource, util
-from twisted.internet import defer, interfaces, error
-from twisted.web import http
-from twisted.protocols import loopback
-from twisted.python import log, reflect
-from twisted.internet.address import IPv4Address
 from zope.interface import implements
 
-class DummyRequest:
-    uri='http://dummy/'
-    method = 'GET'
+from twisted.trial import unittest
+from twisted.internet.address import IPv4Address
+from twisted.internet.defer import Deferred
+from twisted.web import server, resource, util
+from twisted.internet import defer, interfaces, error, task
+from twisted.web import http, http_headers
+from twisted.python import log
 
-    def getHeader(self, h):
-        return None
+
+class DummyRequest:
+    """
+    Represents a dummy or fake request.
+
+    @ivar _finishedDeferreds: C{None} or a C{list} of L{Deferreds} which will
+        be called back with C{None} when C{finish} is called or which will be
+        errbacked if C{processingFailed} is called.
+
+    @type headers: C{dict}
+    @ivar headers: A mapping of header name to header value for all request
+        headers.
+
+    @type outgoingHeaders: C{dict}
+    @ivar outgoingHeaders: A mapping of header name to header value for all
+        response headers.
+
+    @type responseCode: C{int}
+    @ivar responseCode: The response code which was passed to
+        C{setResponseCode}.
+
+    @type written: C{list} of C{str}
+    @ivar written: The bytes which have been written to the request.
+    """
+    uri = 'http://dummy/'
+    method = 'GET'
+    client = None
 
     def registerProducer(self, prod,s):
         self.go = 1
@@ -28,6 +53,7 @@ class DummyRequest:
     def unregisterProducer(self):
         self.go = 0
 
+
     def __init__(self, postpath, session=None):
         self.sitepath = []
         self.written = []
@@ -38,6 +64,25 @@ class DummyRequest:
         self.protoSession = session or server.Session(0, self)
         self.args = {}
         self.outgoingHeaders = {}
+        self.responseHeaders = http_headers.Headers()
+        self.responseCode = None
+        self.headers = {}
+        self._finishedDeferreds = []
+
+
+    def getHeader(self, name):
+        """
+        Retrieve the value of a request header.
+
+        @type name: C{str}
+        @param name: The name of the request header for which to retrieve the
+            value.  Header names are compared case-insensitively.
+
+        @rtype: C{str} or L{NoneType}
+        @return: The value of the specified request header.
+        """
+        return self.headers.get(name.lower(), None)
+
 
     def setHeader(self, name, value):
         """TODO: make this assert on write() if the header is content-length
@@ -50,24 +95,81 @@ class DummyRequest:
         assert not self.written, "Session cannot be requested after data has been written."
         self.session = self.protoSession
         return self.session
+
     def write(self, data):
         self.written.append(data)
+
+    def notifyFinish(self):
+        """
+        Return a L{Deferred} which is called back with C{None} when the request
+        is finished.  This will probably only work if you haven't called
+        C{finish} yet.
+        """
+        finished = Deferred()
+        self._finishedDeferreds.append(finished)
+        return finished
+
+
     def finish(self):
+        """
+        Record that the request is finished and callback and L{Deferred}s
+        waiting for notification of this.
+        """
         self.finished = self.finished + 1
+        if self._finishedDeferreds is not None:
+            observers = self._finishedDeferreds
+            self._finishedDeferreds = None
+            for obs in observers:
+                obs.callback(None)
+
+
+    def processingFailed(self, reason):
+        """
+        Errback and L{Deferreds} waiting for finish notification.
+        """
+        if self._finishedDeferreds is not None:
+            observers = self._finishedDeferreds
+            self._finishedDeferreds = None
+            for obs in observers:
+                obs.errback(reason)
+
+
     def addArg(self, name, value):
         self.args[name] = [value]
+
+
     def setResponseCode(self, code):
-        assert not self.written, "Response code cannot be set after data has been written: %s." % string.join(self.written, "@@@@")
+        """
+        Set the HTTP status response code, but takes care that this is called
+        before any data is written.
+        """
+        assert not self.written, "Response code cannot be set after data has been written: %s." % "@@@@".join(self.written)
+        self.responseCode = code
+
+
     def setLastModified(self, when):
-        assert not self.written, "Last-Modified cannot be set after data has been written: %s." % string.join(self.written, "@@@@")
+        assert not self.written, "Last-Modified cannot be set after data has been written: %s." % "@@@@".join(self.written)
+
+
     def setETag(self, tag):
-        assert not self.written, "ETag cannot be set after data has been written: %s." % string.join(self.written, "@@@@")
+        assert not self.written, "ETag cannot be set after data has been written: %s." % "@@@@".join(self.written)
+
+
+    def getClientIP(self):
+        """
+        Return the IPv4 address of the client which made this request, if there
+        is one, otherwise C{None}.
+        """
+        if isinstance(self.client, IPv4Address):
+            return self.client.host
+        return None
+
 
 class ResourceTestCase(unittest.TestCase):
     def testListEntities(self):
         r = resource.Resource()
         self.failUnlessEqual([], r.listEntities())
-        
+
 
 class SimpleResource(resource.Resource):
     def render(self, request):
@@ -77,33 +179,127 @@ class SimpleResource(resource.Resource):
         else:
             return "correct"
 
+
+class DummyChannel:
+    class TCP:
+        port = 80
+
+        def __init__(self):
+            self.written = StringIO()
+
+        def getPeer(self):
+            return IPv4Address("TCP", '192.168.1.1', 12344)
+
+        def write(self, bytes):
+            assert isinstance(bytes, str)
+            self.written.write(bytes)
+
+        def writeSequence(self, iovec):
+            map(self.write, iovec)
+
+        def getHost(self):
+            return IPv4Address("TCP", '10.0.0.1', self.port)
+
+    class SSL(TCP):
+        implements(interfaces.ISSLTransport)
+
+    site = server.Site(resource.Resource())
+
+    def __init__(self):
+        self.transport = self.TCP()
+
+
+    def requestDone(self, request):
+        pass
+
+
+
+
 class SiteTest(unittest.TestCase):
-    def testSimplestSite(self):
+    def test_simplestSite(self):
+        """
+        L{Site.getResourceFor} returns the C{""} child of the root resource it
+        is constructed with when processing a request for I{/}.
+        """
         sres1 = SimpleResource()
         sres2 = SimpleResource()
         sres1.putChild("",sres2)
         site = server.Site(sres1)
-        assert site.getResourceFor(DummyRequest([''])) is sres2, "Got the wrong resource."
+        self.assertIdentical(
+            site.getResourceFor(DummyRequest([''])),
+            sres2, "Got the wrong resource.")
 
 
 
 class SessionTest(unittest.TestCase):
 
     def setUp(self):
+        """
+        Set up a session using a simulated scheduler. Creates a
+        C{times} attribute which specifies the return values of the
+        session's C{_getTime} method.
+        """
+        clock = self.clock = task.Clock()
+        times = self.times = []
+
+        class MockSession(server.Session):
+            """
+            A mock L{server.Session} object which fakes out scheduling
+            with the C{clock} attribute and fakes out the current time
+            to be the elements of L{SessionTest}'s C{times} attribute.
+            """
+            def loopFactory(self, *a, **kw):
+                """
+                Create a L{task.LoopingCall} which uses
+                L{SessionTest}'s C{clock} attribute.
+                """
+                call = task.LoopingCall(*a, **kw)
+                call.clock = clock
+                return call
+
+            def _getTime(self):
+                return times.pop(0)
+
         self.site = server.Site(SimpleResource())
+        self.site.sessionFactory = MockSession
+
+
+    def test_basicExpiration(self):
+        """
+        Test session expiration: setup a session, and simulate an expiration
+        time.
+        """
+        self.times.extend([0, server.Session.sessionTimeout + 1])
+        session = self.site.makeSession()
+        hasExpired = [False]
+        def cbExpire():
+            hasExpired[0] = True
+        session.notifyOnExpire(cbExpire)
+        self.clock.advance(server.Site.sessionCheckTime - 1)
+        # Looping call should not have been executed
+        self.failIf(hasExpired[0])
+
+        self.clock.advance(1)
+
+        self.failUnless(hasExpired[0])
+
 
     def test_delayedCallCleanup(self):
-        """Checking to make sure Sessions do not leave extra DelayedCalls.
         """
-        from twisted.internet import reactor
-        delayedCallsBeforeSession = repr(reactor.getDelayedCalls())
+        Checking to make sure Sessions do not leave extra DelayedCalls.
+        """
+        self.times.extend([0, 100])
 
         session = self.site.makeSession()
+        loop = session.checkExpiredLoop
         session.touch()
+        self.failUnless(loop.running)
+
         session.expire()
 
-        self.failUnlessEqual(delayedCallsBeforeSession,
-                             repr(reactor.getDelayedCalls()))
+        self.failIf(self.clock.calls)
+        self.failIf(loop.running)
+
 
 
 # Conditional requests:
@@ -153,7 +349,7 @@ class ConditionalTest(unittest.TestCase):
         for l in ["GET / HTTP/1.1",
                   "Accept: text/html"]:
             self.channel.lineReceived(l)
-    
+
     def tearDown(self):
         self.channel.connectionLost(None)
 
@@ -261,18 +457,6 @@ resource = Data('dynamic world','text/plain')
         child_without_ext = f.getChild('AreBelong', dreq)
         self.assertNotEquals(child_without_ext, f.childNotFound)
 
-class DummyChannel:
-    class TCP:
-        port = 80
-        def getPeer(self):
-            return IPv4Address("TCP", 'client.example.com', 12344)
-        def getHost(self):
-            return IPv4Address("TCP", 'example.com', self.port)
-    class SSL(TCP):
-        implements(interfaces.ISSLTransport)
-    transport = TCP()
-    site = server.Site(resource.Resource())
-
 class TestRequest(unittest.TestCase):
 
     def testChildLink(self):
@@ -294,7 +478,6 @@ class TestRequest(unittest.TestCase):
 
     def testPrePathURLNonDefault(self):
         d = DummyChannel()
-        d.transport = DummyChannel.TCP()
         d.transport.port = 81
         request = server.Request(d, 1)
         request.setHost('example.com', 81)
@@ -304,7 +487,6 @@ class TestRequest(unittest.TestCase):
 
     def testPrePathURLSSLPort(self):
         d = DummyChannel()
-        d.transport = DummyChannel.TCP()
         d.transport.port = 443
         request = server.Request(d, 1)
         request.setHost('example.com', 443)
@@ -344,7 +526,6 @@ class TestRequest(unittest.TestCase):
 
     def testPrePathURLSetSSLHost(self):
         d = DummyChannel()
-        d.transport = DummyChannel.TCP()
         d.transport.port = 81
         request = server.Request(d, 1)
         request.setHost('foo.com', 81, 1)
@@ -352,9 +533,22 @@ class TestRequest(unittest.TestCase):
         request.requestReceived('GET', '/foo/bar', 'HTTP/1.0')
         self.assertEqual(request.prePathURL(), 'https://foo.com:81/foo/bar')
 
+
+    def test_prePathURLQuoting(self):
+        """
+        L{Request.prePathURL} quotes special characters in the URL segments to
+        preserve the original meaning.
+        """
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.setHost('example.com', 80)
+        request.gotLength(0)
+        request.requestReceived('GET', '/foo%2Fbar', 'HTTP/1.0')
+        self.assertEqual(request.prePathURL(), 'http://example.com/foo%2Fbar')
+
+
     def testNotifyFinishConnectionLost(self):
         d = DummyChannel()
-        d.transport = DummyChannel.TCP()
         request = server.Request(d, 1)
         finished = request.notifyFinish()
         request.connectionLost(error.ConnectionDone("Connection done"))
@@ -372,7 +566,6 @@ class RootResource(resource.Resource):
 class RememberURLTest(unittest.TestCase):
     def createServer(self, r):
         chan = DummyChannel()
-        chan.transport = DummyChannel.TCP()
         chan.site = server.Site(r)
         return chan
 
@@ -416,7 +609,6 @@ class NewRenderTestCase(unittest.TestCase):
     def _getReq(self):
         d = DummyChannel()
         d.site.resource.putChild('newrender', NewRenderResource())
-        d.transport = DummyChannel.TCP()
         d.transport.port = 81
         request = server.Request(d, 1)
         request.setHost('example.com', 81)
@@ -464,22 +656,17 @@ class SDTest(unittest.TestCase):
         resource.getChildForRequest(s, d)
         self.assertEqual(d.postpath, ['bar', 'baz'])
 
+
+
 class DummyRequestForLogTest(DummyRequest):
-    uri='/dummy' # parent class uri has "http://", which doesn't really happen
+    uri = '/dummy' # parent class uri has "http://", which doesn't really happen
     code = 123
-    client = '1.2.3.4'
+
     clientproto = 'HTTP/1.0'
     sentLength = None
+    client = IPv4Address('TCP', '1.2.3.4', 12345)
 
-    def __init__(self, *a, **kw):
-        DummyRequest.__init__(self, *a, **kw)
-        self.headers = {}
 
-    def getHeader(self, h):
-        return self.headers.get(h.lower(), None)
-
-    def getClientIP(self):
-        return self.client
 
 class TestLogEscaping(unittest.TestCase):
     def setUp(self):
