@@ -1,20 +1,12 @@
 # -*- test-case-name: twisted.web.test.test_xmlrpc -*-
-#
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 A generic resource for publishing objects via XML-RPC.
 
-Requires xmlrpclib (comes standard with Python 2.2 and later, otherwise can be
-downloaded from http://www.pythonware.com/products/xmlrpc/).
-
-Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
+Maintainer: Itamar Shtull-Trauring
 """
-from __future__ import nested_scopes
-
-__version__ = "$Revision: 1.32 $"[11:-2]
 
 # System Imports
 import xmlrpclib
@@ -312,24 +304,25 @@ class _QueryFactory(protocol.ClientFactory):
         if not self.deferred:
             return
         try:
-            response = xmlrpclib.loads(contents)
+            response = xmlrpclib.loads(contents)[0][0]
         except:
-            self.deferred.errback(failure.Failure())
-            self.deferred = None
+            deferred, self.deferred = self.deferred, None
+            deferred.errback(failure.Failure())
         else:
-            self.deferred.callback(response[0][0])
-            self.deferred = None
+            deferred, self.deferred = self.deferred, None
+            deferred.callback(response)
 
     def clientConnectionLost(self, _, reason):
         if self.deferred is not None:
-            self.deferred.errback(reason)
-            self.deferred = None
+            deferred, self.deferred = self.deferred, None
+            deferred.errback(reason)
 
     clientConnectionFailed = clientConnectionLost
 
     def badStatus(self, status, message):
-        self.deferred.errback(ValueError(status, message))
-        self.deferred = None
+        deferred, self.deferred = self.deferred, None
+        deferred.errback(ValueError(status, message))
+
 
 
 class Proxy:
@@ -398,6 +391,14 @@ class Proxy:
         self.allowNone = allowNone
 
     def callRemote(self, method, *args):
+        """
+        Call remote XML-RPC C{method} with given arguments.
+
+        @return: a L{defer.Deferred} that will fire with the method response,
+            or a failure if the method failed. Generally, the failure type will
+            be L{Fault}, but you can also have an C{IndexError} on some buggy
+            servers giving empty responses.
+        """
         factory = self.queryFactory(
             self.path, self.host, method, self.user,
             self.password, self.allowNone, args)
@@ -409,5 +410,8 @@ class Proxy:
             reactor.connectTCP(self.host, self.port or 80, factory)
         return factory.deferred
 
-__all__ = ["XMLRPC", "Handler", "NoSuchFunction", "Fault", "Proxy"]
 
+__all__ = [
+    "XMLRPC", "Handler", "NoSuchFunction", "Proxy",
+
+    "Fault", "Binary", "Boolean", "DateTime"]
